@@ -56,30 +56,56 @@ buffer_info_t* buffer_new(int id, long rate){
 	buf->error = 0;
 	buf->percent_filled = 0.;
 	buf->normalized = 0;
+	buf->snap_offset = 0;
 	buf->volume_factor = 1.;
 	buf->filename= NULL;
 	buf->shortname = NULL;
 	buf->chunk_list = NULL;
+//	printf ("buffer get initial group\n");
 	buffer_get_group(buf, DS_PRIORITY_NORMAL);
 	pthread_mutex_init (&buf->buffermutex, NULL);
 
 	return buf;
 }
 
-static void buffer_delete_group(buffer_info_t* buf, int gid){
+void buffer_delete_group(buffer_info_t* buf, int gid){
+/*	int i;*/
+	if (!buf) return;
         if (!buf->chunk_list) return;
+	if (buf->group_max < gid) return;
         if (!buf->chunk_list[gid]) return;
 
-        chunkgroup_delete(buf->chunk_list[gid]);
-        buf->chunk_list[gid] = NULL;
+	/* delete only when DISCSTREAM */
+	if (gid > 1){
+        	chunkgroup_delete(buf->chunk_list[gid]);
+        	buf->chunk_list[gid] = NULL;	
+	}
+/*
+	printf ("deleting group %d from %s      -> ",gid, buf->shortname);
+	printf ("state now: ");
+	for (i = 0; i <=buf->group_max; i++){
+		if (buf->chunk_list[i]) printf ("%d, ",i);
+	}
+	printf ("\n");
+*/
 }
 
 int buffer_delete(buffer_info_t* buf){
 	int i;
 
-	for (i = 0; i <= buf->group_max; i++)
+	/* LATER: care for filechunks !*/
+	for (i = buf->group_max; i > 0; i--)
 		buffer_delete_group(buf, i);
 
+	/* delete the initial group: */
+	if (buf->chunk_list[0]) {
+		chunkgroup_delete(buf->chunk_list[0]);
+		buf->chunk_list[0] = NULL;
+	}
+
+	free (buf->chunk_list);
+	buf->chunk_list = NULL;
+
 	buf->pending = BUFFER_STATUS_DEAD;
 	if (buf->filename) free (buf->filename);
 	buf->filename = NULL;
@@ -100,18 +126,25 @@ int buffer_get_type(buffer_info_t* buf){
 void buffer_set_area(buffer_info_t* buf, int group, long start, long end, long bandwith){
 	if (!buf->chunk_list) return;
         if (!buf->chunk_list[group]) return;
+
+	if (start < 0) start = 0;
+	if (start > buffer_get_size(buf)) start = buffer_get_size(buf);
+	if (end < 0) end = 0;
+        if (end > buffer_get_size(buf)) end = buffer_get_size(buf);
 	
 	start = (long)((float)start * buf->samplerate_factor);
 	end   = (long)((float)end   * buf->samplerate_factor);
 
-/*	printf ("area set: %ld - %ld\n",start,end);*/
-
 	if (start > end){
 		long tmp = start;
 		start = end;
 		end = tmp;
 
 	}
+/*
+	printf ("area set: %ld - %ld of %ld, bandwidth:%ld\n",
+			start,end, buffer_get_size(buf),bandwith);
+*/
 	buf->chunk_list[group]->area_start = start;
 	buf->chunk_list[group]->area_end = end;
 	buf->chunk_list[group]->area_bandwith = bandwith;	
@@ -120,16 +153,14 @@ void buffer_set_area(buffer_info_t* buf, int group, long start, long end, long b
 static int buffer_append_group(buffer_info_t* buf, int priority){
 	chunk_group_t* group = chunkgroup_new(++buf->group_max, priority);
 
-	buf->chunk_list = realloc(buf->chunk_list, sizeof(chunk_group_t) * (buf->group_max + 1));
+	buf->chunk_list = realloc(buf->chunk_list, sizeof(chunk_group_t) * (buf->group_max));
 	buf->chunk_list[buf->group_max] = group;
-/*	printf ("appended group nr.%d\n",buf->group_max);*/
-
 	return buf->group_max;
 }
 
 int buffer_get_group(buffer_info_t* buf, int priority){
 	if (buf->type == BUFFER_TYPE_DISCSTREAM){
-		int i = buffer_append_group(buf, DS_PRIORITY_NORMAL);
+		int i = buffer_append_group(buf, priority);
 		buf->chunk_list[i]->priority = priority;
 		return i;
 	}else{ 
@@ -142,13 +173,14 @@ int buffer_get_group(buffer_info_t* buf, int priority){
 
 int buffer_get_channels(buffer_info_t* buf){
 	if (!buf->chunk_list) return 0;
+	if (!buf->chunk_list[0]) return 0;
 	if (!buf->chunk_list[0]->readfc) return 0;
 
 	return filechunk_get_channels(buf->chunk_list[0]->readfc);
 }
 
 /* feedback functions: */
-/*
+
 float buffer_get_sampleratefactor(buffer_info_t* buf, long srate){
 	if (!buf) return 0;
 	if (!buf->chunk_list) return 0;
@@ -158,26 +190,52 @@ float buffer_get_sampleratefactor(buffer_info_t* buf, long srate){
 		return ((float)filechunk_get_samplerate(buf->chunk_list[0]->readfc) / (float)srate ) ;
 	else return 1.;
 }
-*/
+
 
 void buffer_set_channels(buffer_info_t* buf, int channels){
-#if 0
-	if (buf->channels == channels) return;
+	if (!buf) return;
+	if (!buf->chunk_list[0]) return;
 	if (buf->type != BUFFER_TYPE_MEMORY) return;
-	
-	if (channels == 1){
-/*		printf ("1 channel\n");*/
-		free(buf->R);
-		buf->R = NULL;
-	}else if (channels == 2)
-		buf->R = (float*) realloc(buf->R, sizeof(float) * buffer_get_size(buf));
-	
-	buf->channels = channels;
-#endif
+	if (filechunk_get_channels(buf->chunk_list[0]->readfc) == channels) return;
+
+/*	printf ("%s set channels to %d\n",buf->shortname, channels);	*/
+	if ((channels > 0) && (channels < 3)){
+		int gid;
+		for (gid = 0; gid <= buf->group_max; gid++){
+			if (buf->chunk_list[gid]){ 
+				filechunk_set_channels(buf->chunk_list[gid]->readfc, channels);
+				filechunk_set_channels(buf->chunk_list[gid]->prevfc, channels);
+				filechunk_set_channels(buf->chunk_list[gid]->nextfc, channels);
+				filechunk_set_channels(buf->chunk_list[gid]->wrapfc, channels);
+			}
+		}
+	}
+}
+
+/* not in use:
+void buffer_set_snap_offset(buffer_info_t* buf, long offset){
+	if (buf) {
+		if (offset < 0) offset = 0;
+        	if (offset > buffer_get_size(buf)) offset = buffer_get_size(buf);
+		buf->snap_offset  = offset;
+	}
+}
+
+long buffer_get_snap_offset(buffer_info_t* buf){
+	if (buf) return buf->snap_offset;
+	return 0;
+}
+*/
+
+int buffer_get_status(buffer_info_t* buf){
+	if (!buf) return 0;
+	return buf->status;
 }
 
 long buffer_get_size(buffer_info_t* buf){
+	if (!buf) return 0;
 	if (!buf->chunk_list) return 0;
+	if (!buf->chunk_list[0]) return 0;
         if (!buf->chunk_list[0]->readfc) return 0;
 
 	if (buf->type == BUFFER_TYPE_DISCSTREAM)
@@ -204,6 +262,13 @@ int is_inside (filechunk_t* fc, long index) {
 	if (filechunk_get_start(fc) > index) return 0;
 	return 1;
 }
+
+int is_inside_done (filechunk_t* fc, long index) {
+        if (filechunk_get_done(fc)  < index) return 0;
+        if (filechunk_get_start(fc) > index) return 0;
+        return 1; 
+}
+
 static void shift_right(chunk_group_t * cg){
 	filechunk_t* tempfc = cg->readfc;
         cg->readfc = cg->nextfc;
@@ -224,6 +289,7 @@ static float valuehelper(filechunk_t* fc, int channel, long index){
 		long l = 0;
 		while (((filechunk_get_done(fc) + filechunk_get_start(fc)) 
 			< index) && (l++ < 1000)){
+/*				printf ("sleepwait\n");*/
                                 tv.tv_sec = 0;
                                 tv.tv_usec = 100;
                                 select(0, NULL, NULL, NULL, &tv);
@@ -245,27 +311,37 @@ static long wrapindex (long index, long start, long end){
 float buffer_readsample_interpolated(buffer_info_t* buf, int channel, float index, int group){
 	if (!buf) return 0.;
 	if (!buf->chunk_list) return 0;
-	if (buf->group_max < group) return 0;
-/*	printf ("%d%d",group,(int)buf->chunk_list[group]->readfc);*/
+	if (!buf->chunk_list[group]) return 0;
 
-	index = buf->samplerate_factor * index;
+	index *= buf->samplerate_factor;
 
-        if (!buf->chunk_list[group]->readfc){
-		buf->chunk_list[group]->readfc = filechunk_new(buf->filename);
-		if (buf->type == BUFFER_TYPE_DISCSTREAM){
-			buf->chunk_list[group]->nextfc = filechunk_new(buf->filename);
-			buf->chunk_list[group]->prevfc = filechunk_new(buf->filename);
-			buf->chunk_list[group]->wrapfc = filechunk_new(buf->filename);
-		}
-	}
 	if (buf->type == BUFFER_TYPE_DISCSTREAM){ 
+		if (buf->group_max < group) return 0;
+
+		if (!buf->chunk_list[group]->readfc){
+                	buf->chunk_list[group]->readfc = filechunk_new(buf->filename);
+                	if (buf->type == BUFFER_TYPE_DISCSTREAM){
+/*				printf ("make new filechunks for %s, group %d\n", buf->shortname, group);*/
+                        	buf->chunk_list[group]->nextfc = filechunk_new(buf->filename);
+                        	buf->chunk_list[group]->prevfc = filechunk_new(buf->filename);
+                        	buf->chunk_list[group]->wrapfc = filechunk_new(buf->filename);
+                	}
+        	}
 		if ( is_inside(buf->chunk_list[group]->readfc,index)){
 			return valuehelper(buf->chunk_list[group]->readfc,channel,index) * buf->volume_factor;
 		}else{
 			chunk_group_t* cg = buf->chunk_list[group];
+/*
+			printf ("not inside index:%lf.0, this=%ld-%ld, next=%ld-%ld, prev=%ld-%ld, wrap=%ld-%ld\n",
+				index,
+                                filechunk_get_start(cg->readfc),filechunk_get_end(cg->readfc),
+                                filechunk_get_start(cg->nextfc),filechunk_get_end(cg->nextfc),
+                                filechunk_get_start(cg->prevfc),filechunk_get_end(cg->prevfc),
+                                filechunk_get_start(cg->wrapfc),filechunk_get_end(cg->wrapfc));
+*/
 			long goodsize = cg->area_bandwith * 2;
-			if (goodsize <  ds_buffersize) goodsize = ds_buffersize;
-		        if (	!is_inside(cg->readfc,index) && is_inside(cg->nextfc,index) ) {
+			if (goodsize < ds_buffersize) goodsize = ds_buffersize;
+		        if (!is_inside(cg->readfc,index) && is_inside(cg->nextfc,index) ) {
 				long wi = (long)index - cg->area_bandwith;
 				wi = wrapindex(wi, cg->area_start, 
 					cg->area_end ? cg->area_end : filechunk_get_filelength (cg->readfc));
@@ -274,6 +350,14 @@ float buffer_readsample_interpolated(buffer_info_t* buf, int channel, float inde
 					return valuehelper(cg->nextfc,channel,index) * buf->volume_factor;
 				}else{
 					/* not still in acceptable range */
+/*
+					printf ("next index:%.0lf, this=%ld-%ld, next=%ld-%ld, prev=%ld-%ld, wrap=%ld-%ld\n",
+                                		index,
+                                		filechunk_get_start(cg->readfc),filechunk_get_end(cg->readfc),
+                                		filechunk_get_start(cg->nextfc),filechunk_get_end(cg->nextfc),
+                                		filechunk_get_start(cg->prevfc),filechunk_get_end(cg->prevfc),
+                                		filechunk_get_start(cg->wrapfc),filechunk_get_end(cg->wrapfc));
+*/
 					long s,e;
                                		buffer_lock(buf);
 					shift_right(cg);
@@ -282,6 +366,7 @@ float buffer_readsample_interpolated(buffer_info_t* buf, int channel, float inde
                                		if (e > filechunk_get_filelength(cg->readfc))
                                        		e = filechunk_get_filelength(cg->readfc);
 					if (cg->area_end && (s >= cg->area_end)){
+/*						printf ("wrap\n");*/
 						s = cg->area_start;
                                                 e = s + goodsize;
 						if (e > filechunk_get_filelength(cg->readfc))
@@ -291,8 +376,10 @@ float buffer_readsample_interpolated(buffer_info_t* buf, int channel, float inde
 						long we = ws + goodsize;
 						if (we > filechunk_get_filelength(cg->readfc))
 							we = filechunk_get_filelength(cg->readfc);
+/*						printf ("set read\n");*/
 						filechunk_set(cg->wrapfc,ws, we, cg->priority, group);
 					}
+/*					printf ("set next, start:%ld, area_end:%ld \n",s,cg->area_end);*/
                                		filechunk_set(cg->nextfc,s, e, cg->priority, group);
                                		buffer_unlock(buf);
 					return valuehelper(cg->readfc,channel,index) * buf->volume_factor;
@@ -307,6 +394,7 @@ float buffer_readsample_interpolated(buffer_info_t* buf, int channel, float inde
 					return valuehelper(cg->prevfc,channel,index) * buf->volume_factor;
 				}else{
                                         /* not still in acceptable range */
+/*					printf ("previous\n");*/
                     	 		long s,e;
                                		buffer_lock(buf);
 					shift_left(cg);
@@ -327,12 +415,21 @@ float buffer_readsample_interpolated(buffer_info_t* buf, int channel, float inde
                                		buffer_unlock(buf);
 					return valuehelper(cg->readfc,channel,index) * buf->volume_factor;
 				}
-			}else if ( !is_inside(cg->readfc,index) && is_inside(cg->wrapfc,index)) {
+			}else if ( !is_inside(cg->readfc,index) && is_inside_done(cg->wrapfc,index)) {
 				return valuehelper(cg->wrapfc,channel,index) * buf->volume_factor;
+				/* should be more sophisticated here */
                 	}else{
+/*
+				printf ("different: index:%.0lf this=%ld-%ld, next=%ld-%ld, prev=%ld-%ld, wrap=%ld-%ld\n",
+				index,
+				filechunk_get_start(cg->readfc),filechunk_get_end(cg->readfc),
+				filechunk_get_start(cg->nextfc),filechunk_get_end(cg->nextfc),
+				filechunk_get_start(cg->prevfc),filechunk_get_end(cg->prevfc),
+				filechunk_get_start(cg->wrapfc),filechunk_get_end(cg->wrapfc));
+*/
 				long s,e;
-				s = (long)index - (ds_buffersize / 2);
-				e = (long)index + (ds_buffersize / 2);
+				s = (long)index - (goodsize / 2);
+				e = (long)index + (goodsize / 2);
 				if (s < 0) s = 0;
 				if (e > filechunk_get_filelength(cg->readfc)) 
 					e = filechunk_get_filelength(cg->readfc);
@@ -341,22 +438,33 @@ float buffer_readsample_interpolated(buffer_info_t* buf, int channel, float inde
 					filechunk_set(cg->readfc,s, e, cg->priority, group);
 				else	filechunk_set(cg->readfc,s, e, DS_PRIORITY_URGENT, group);
 				s += goodsize;
-				e = s + goodsize;
+				if (s < cg->area_end)
+					e = s + goodsize;
+				else {
+					s = cg->area_start;
+					e = s + goodsize;
+				}
 				filechunk_set(cg->nextfc, s, e, cg->priority, group);
+
 				s -= goodsize * 2;
-				if (s < 0) s = 0;
 				e = s + goodsize;
+				if (s < cg->area_start){
+					e = cg->area_end;
+					s -= goodsize;	
+				}
 				filechunk_set(cg->prevfc, s, e, cg->priority, group);
 				buffer_unlock(buf);
 
 				if (cg->priority == DS_PRIORITY_RELAXED)
 					return valuehelper(cg->readfc,channel,index) * buf->volume_factor;
+
+/*				printf ("sample = zero because cpu = too slow\n");*/
 				return 0.;
 			}
 		}
 	} else {
 		/* LATER: wrap interpolation around loopborders */
-/*		if (!buf->chunk_list[group]->readfc) return 0.;*/
+		/* this is quite expensive: */
 		if (channel == 1)
 			return inter4pol_float(buf->chunk_list[group]->readfc->L,
 					index,buffer_get_size(buf) / buf->samplerate_factor) * buf->volume_factor;
@@ -373,22 +481,36 @@ void buffer_writesample(buffer_info_t* buf, int channel, long index, int group,
 	if (!buf->chunk_list[0]->readfc) return;
 	if (buf->type == BUFFER_TYPE_DISCSTREAM) return;  /* never write to a discstream */
 
-        index = (long)((float)index * buf->samplerate_factor);
-
+        index *= buf->samplerate_factor;
+	
 	filechunk_write_sample(buf->chunk_list[group]->readfc, channel, index, sample);
 }
 
+void buffer_write_sampleblock(buffer_info_t* buf, long index, long size, int group, float* Lbuf, float* Rbuf){
+	if (!buf) return;
+	if (!buf->chunk_list) return;
+	if (!buf->group_max < group) return;
+	if (!buf->chunk_list[0]->readfc) return;
+	if (buf->type == BUFFER_TYPE_DISCSTREAM) return;  /* never write to a discstream */
+
+	index *= buf->samplerate_factor;
+
+	filechunk_write_sampleblock(buf->chunk_list[group]->readfc, index, size, Lbuf, Rbuf);
+}
+
 float buffer_analyze(buffer_info_t* buf, int gid, long start,long end){
-	int group = gid;
+//	int group = gid;
 	struct timeval tv;
 	float val = .0;
 	float max = .0;
 	long  l;
+	int group;
 
 	if (!buf) return 0.;
         if (!buf->chunk_list) return 0;
 	if (buf->type != BUFFER_TYPE_DISCSTREAM) group = 0;
-
+	group = buffer_get_group (buf, DS_PRIORITY_RELAXED);
+        buffer_set_area(buf, group, start, end, 32767);
 /*
         start = (long)((float)start * buf->samplerate_factor);
         end   = (long)((float)end * buf->samplerate_factor);
@@ -414,6 +536,8 @@ float buffer_analyze(buffer_info_t* buf, int gid, long start,long end){
 			select(0, NULL, NULL, NULL, &tv);
 		}
 	}
+	buffer_delete_group(buf, group);
+//	printf ("max:%f\n",max);
 	return max;
 }
 
@@ -422,11 +546,15 @@ void buffer_normalize(buffer_info_t* buf, float factor, long start, long end){
 	long l;
 	struct timeval tv;
 	float val;
+	int group; 
 
 	if (!buf) return;
         if (!buf->chunk_list) return;
 	if (buf->type == BUFFER_TYPE_DISCSTREAM) return;
 
+	group = buffer_get_group (buf, DS_PRIORITY_RELAXED);
+        buffer_set_area(buf, group, start, end, 32767);
+
 /*
         start = (long)((float)start * buf->samplerate_factor);
         end   = (long)((float)end * buf->samplerate_factor);
@@ -441,11 +569,11 @@ void buffer_normalize(buffer_info_t* buf, float factor, long start, long end){
 	buffer_lock(buf);
 
 	for (l = start; l <= end; l++){
-		val = buffer_readsample_interpolated(buf,1,(float)l,0) * factor;
+		val = buffer_readsample_interpolated(buf,1,(float)l,group) * factor;
 		buffer_writesample(buf,1,(float)l,0,val);
 		if (buffer_get_channels(buf) > 1){
-			val = buffer_readsample_interpolated(buf,2,(float)l,0) * factor;
-                	buffer_writesample(buf,2,(float)l,0,val);
+			val = buffer_readsample_interpolated(buf,2,(float)l,group) * factor;
+                	buffer_writesample(buf,2,(float)l,group,val);
 		}
 		if (!(l % 100000)){
                         tv.tv_sec = 0;
@@ -456,7 +584,9 @@ void buffer_normalize(buffer_info_t* buf, float factor, long start, long end){
                 }
 	}
 	buf->normalized = 1;
+/*	printf ("%s normalized:%d\n",buf->shortname, buf->normalized);*/
 	buffer_unlock(buf);
+	buffer_delete_group (buf, group);
 }
 
 void buffer_set_volume_factor(buffer_info_t* buf, float factor){
@@ -506,7 +636,10 @@ static filechunk_t* buffer_order_filechunk(buffer_info_t* buf, long start, long
 		return NULL;
 	}
 	if (end < 0) end = filechunk_get_length(fc);
-	filechunk_set(fc, start, end, priority, group);		
+	if (!filechunk_set(fc, start, end, priority, group)){
+		filechunk_delete(fc);
+		return NULL;		
+	}
 
 	return fc;
 }
@@ -533,44 +666,52 @@ void buffer_open_discstream (buffer_info_t* buf, char* fname, long srate){
 /*	long s,l; */
 	filechunk_t* fc;
 	char* sn;
+	float min,sec;
 
 	buf->type = BUFFER_TYPE_DISCSTREAM;
 	buf->filename = (char*) malloc (sizeof(char) * (strlen(fname) + 1));
         strcpy(buf->filename,fname);
-	lp = strnflst   (fname,"/",-1);
-        sn = calloc(sizeof(char), (strlen(fname) - lp + 5));
-	snprintf (sn,strlen(fname) - lp + 4,"DS: %s",(char*)(buf->filename + lp + 1));
-	buffer_set_shortname(buf,sn);
-	fc = buffer_order_filechunk(buf,0,1,DS_PRIORITY_URGENT,0);
-	buf->chunk_list[0]->readfc = fc;
-	buf->pending = BUFFER_STATUS_READY;
-	buf->samplerate_factor = (float)filechunk_get_samplerate(fc) / (float)buf->parent_samplerate;
+	if ((fc = buffer_order_filechunk(buf,0,1,DS_PRIORITY_URGENT,0))){
+		lp = strnflst (fname,"/",-1);
+		sn = calloc(sizeof(char), (strlen(fname) - lp + 16));
+        	sec = (float)filechunk_get_filelength(fc) / (float)filechunk_get_samplerate (fc);
+        	min = (int)(sec / 60.);
+        	sec -= min * 60;
+        	snprintf (sn,strlen(fname) - lp + 16,"DS: %s (%0.0f:%02.0f)",(char*)(buf->filename + lp + 1),min,sec);
+		buffer_set_shortname(buf,sn);
+		buf->chunk_list[0]->readfc = fc;
+		buf->chunk_list[0]->nextfc = filechunk_new(buf->filename);
+		buf->chunk_list[0]->prevfc = filechunk_new(buf->filename);
+		buf->chunk_list[0]->wrapfc = filechunk_new(buf->filename);
+		buf->pending = BUFFER_STATUS_READY;
+		buf->samplerate_factor = (float)filechunk_get_samplerate(fc) / (float)buf->parent_samplerate;
+	}else buf->status = BUFFER_STATUS_DEAD;
 /*	printf ("samplerate factor :%.4f\n",buf->samplerate_factor);*/
 	return;
 }
 
-void  buffer_loadfile(buffer_info_t* buf, char* fname, long srate){
+void buffer_loadfile(buffer_info_t* buf, char* fname, long srate){
 	/* create a reding thread, wait for join, return success*/
 	int lp;
 	filechunk_t* fc;
 	char* sn;
+	float min,sec;
 
 	buf->type = BUFFER_TYPE_FILEBUFFER;
 	buf->filename = (char*) malloc (sizeof(char) * (strlen(fname) + 1));
 	strcpy(buf->filename,fname);
-/*	printf("filename now:%s\n",buf->filename);*/
-	lp = strnflst	(fname,"/",-1);
-	sn = calloc(sizeof(char), (strlen(fname) - lp + 5));
-        snprintf (sn,strlen(fname) - lp + 4,"FB: %s",(char*)(buf->filename + lp + 1));
-        buffer_set_shortname(buf,sn);
-/*	printf ("file to load:%s\n",buf->filename);*/
-	fc = buffer_order_filechunk(buf,0,-1, DS_PRIORITY_NORMAL,0);
-	buf->chunk_list[0]->readfc = fc;
-	buf->pending = BUFFER_STATUS_TOLOAD;
-	buf->samplerate_factor = (float)filechunk_get_samplerate(fc) / (float)buf->parent_samplerate;
-/*	printf ("samplerate factor :%.4f (%f / %f)\n",buf->samplerate_factor);*/
-/*	printf("shortname now:%s\n",buf->shortname);*/
-
+	if ((fc = buffer_order_filechunk(buf,0,-1, DS_PRIORITY_NORMAL,0))){
+		lp = strnflst   (fname,"/",-1);
+        	sn = calloc(sizeof(char), (strlen(fname) - lp + 16));
+        	sec = (float)filechunk_get_filelength(fc) / (float)filechunk_get_samplerate (fc);
+        	min = (int)(sec / 60.);
+        	sec -= min * 60;
+        	snprintf (sn,strlen(fname) - lp + 16,"FB: %s (%0.0f:%02.0f)",(char*)(buf->filename + lp + 1),min,sec);
+        	buffer_set_shortname(buf,sn);
+		buf->chunk_list[0]->readfc = fc;
+		buf->pending = BUFFER_STATUS_TOLOAD;
+		buf->samplerate_factor = (float)filechunk_get_samplerate(fc) / (float)buf->parent_samplerate;
+	}else buf->status = BUFFER_STATUS_DEAD;
 	return;
 }
 
@@ -581,14 +722,13 @@ static int check_fc_disc(filechunk_t *fc, int priority){
 	if (!fc->L) return 0;
 
 /*	if (!fc->L) printf ("!fc->L !!! :l:%ld, done:%ld\n ",fc->length,fc->done); */
-	filechunk_read_chunk(fc);
-	return 1;
+	return filechunk_read_chunk(fc);
 }
 
 static void* buffer_discthread(void *ptr){
 	speciallist_t *buffer_list = (speciallist_t*)ptr;
 	buffer_info_t *buf;
-	int dirty,i,j,k;
+	int dirty,i,j,k,d;
 	
         struct timeval tv;
 
@@ -603,10 +743,17 @@ static void* buffer_discthread(void *ptr){
 				k = 0;
                                 buffer_lock(buf);
 				for (j = 0; j <= buf->group_max; j++){
+					if (buf->chunk_list[j]){
 /*					printf ("j:%d\n",j);*/
-					k += check_fc_disc(buf->chunk_list[j]->readfc, i);
-					k += check_fc_disc(buf->chunk_list[j]->nextfc, i);
-					k += check_fc_disc(buf->chunk_list[j]->prevfc, i);
+						d = check_fc_disc(buf->chunk_list[j]->readfc, i);
+						if (d > 0) k += d;
+						d = check_fc_disc(buf->chunk_list[j]->nextfc, i);
+						if (d < 1) k = 0;
+						d = check_fc_disc(buf->chunk_list[j]->prevfc, i);
+						if (d < 1) k = 0;
+						d = check_fc_disc(buf->chunk_list[j]->wrapfc, i);
+                                        	if (d < 1) k = 0;
+					}
 				}
                                 if (k){ 
 					dirty++;
@@ -616,7 +763,14 @@ static void* buffer_discthread(void *ptr){
 							(double)filechunk_get_done(buf->chunk_list[0]->readfc) /
 							(double)filechunk_get_length(buf->chunk_list[0]->readfc);
 					}
-                                }else if (buf->status != BUFFER_STATUS_READY) buf->pending = BUFFER_STATUS_READY;
+                                }else{
+					if (buf->chunk_list[0]){
+						buf->percent_filled = (double)filechunk_get_done(buf->chunk_list[0]->readfc) /
+                                                        	(double)filechunk_get_length(buf->chunk_list[0]->readfc);
+						if ((buf->percent_filled == 1.) && (buf->status != BUFFER_STATUS_READY)) 
+							buf->pending = BUFFER_STATUS_READY;
+					}
+				}
                                 buffer_unlock(buf);
 				buf = speciallist_get_next(buffer_list,buf);
                 	}
@@ -643,21 +797,18 @@ int  buffer_save (buffer_info_t* buf, char* fname, long srate, long start, long
         SNDFILE*        sf;     
 	sf_count_t	frames;
 	sf_count_t      frames_done = 0;
-	sf_count_t	frames_at_once = 16384;
+	sf_count_t	frames_at_once = 32767;
 	struct timeval tv;
 	float		*tbuf;
 	long i;
+	int channels = buffer_get_channels(buf);
+	int group = buffer_get_group (buf, DS_PRIORITY_RELAXED);
+	buffer_set_area(buf, group, start, end, frames_at_once);
 
 	/* LATER: move this to disc thread */
 
-/*
-        start = (long)((float)start * buf->samplerate_factor);
-        end   = (long)((float)end * buf->samplerate_factor);
-*/
-
-
         tv.tv_sec = 0;
-        tv.tv_usec = 10000;
+        tv.tv_usec = 100000;
 
 	sfinfo->samplerate = srate;
 	sfinfo->channels = buffer_get_channels(buf);
@@ -668,35 +819,41 @@ int  buffer_save (buffer_info_t* buf, char* fname, long srate, long start, long
 
 	if (!sf) return -1;
 	
-	tbuf = malloc (sizeof(float) * frames_at_once * buffer_get_channels(buf));	
+	tbuf = calloc (sizeof(float), frames_at_once * channels);	
 	if (!tbuf) return -1;
 
 	while (frames_done < (end - start)){
-		frames = end - start - frames_done;
+		frames = end - (start + frames_done);
 		if (frames > frames_at_once) frames = frames_at_once;
 
-		for (i = 0; i < frames; i++){
-			*(tbuf + i * buffer_get_channels(buf)) = 
-				buffer_readsample_interpolated(buf, 1,
-					i + start + frames_done,0);
-			if (buffer_get_channels(buf) > 1)
-				*(tbuf + i * buffer_get_channels(buf) + 1) = 
+		if (channels > 1){
+			for (i = 0; i < frames; i++){
+				*(tbuf + i * channels) = 
+					buffer_readsample_interpolated(buf, 1,
+						(float)(start + frames_done + i),group);
+				*(tbuf + i * channels + 1) = 
 					buffer_readsample_interpolated(buf, 2,
-						i + start + frames_done,0);
+						(float)(start + frames_done + i),group);
+			}
+		}else{
+			for (i = 0; i < frames; i++){
+                                *(tbuf + i * channels) =        
+                                        buffer_readsample_interpolated(buf, 1,
+                                                (float)(start + frames_done + i),group);
+			}
 		}
 		frames = sf_writef_float(sf, tbuf, frames);
 		if (frames < 1){
-			printf ("frames != frames\n");
-			free(buf);
+			free(tbuf);
+			if (group) buffer_delete_group(buf, group);
 			return -1;
-		}
+		}/* else printf ("%ld frames written\n",(long)frames);*/
 		frames_done += frames;
-                tv.tv_sec = 0;
-                tv.tv_usec = 10000;
                 select(0, NULL, NULL, NULL, &tv);
 	}
 
 	sf_close(sf);
 	free(tbuf);	
+	if (group) buffer_delete_group(buf, group);
 	return 1;
 }
