@@ -41,6 +41,7 @@ typedef struct _buffer_info {
 	int		normalized;
 	float		volume_factor;
 	long		parent_samplerate;
+	long		snap_offset;
 	float		samplerate_factor;
 	chunk_group_t** chunk_list;
 } buffer_info_t;
@@ -62,6 +63,8 @@ void 		buffer_set_filename(buffer_info_t* buf, char* name);
 void 		buffer_set_shortname(buffer_info_t* buf, char* name);
 char* 		buffer_get_shortname(buffer_info_t* buf);
 int		buffer_get_group(buffer_info_t* buf, int priority);
+void 		buffer_delete_group(buffer_info_t* buf, int gid);
+int		buffer_get_status(buffer_info_t* buf);
 int 		buffer_get_channels(buffer_info_t* buf);
 void 		buffer_set_channels(buffer_info_t* buf, int channels);
 /*void		buffer_set_ds(buffer_info_t* buf, int is_ds);*/
@@ -74,10 +77,15 @@ float 		buffer_analyze(buffer_info_t* buf, int group, long start, long end);
 void		buffer_normalize(buffer_info_t* buf, float factor,long start, long end);
 void		buffer_set_volume_factor(buffer_info_t* buf, float factor);
 float           buffer_get_volume_factor(buffer_info_t* buf);
+/*
+void           	buffer_set_snap_offset(buffer_info_t* buf, long offset);
+long           	buffer_get_snap_offset(buffer_info_t* buf);
+*/
 /*speciallist_t*  buffer_list_new();*/
 void		buffer_discthread_launch(speciallist_t *buffer_list);
 /*float		buffer_readsample(buffer_info_t* buf, int channel, long index, int group);*/
 float 		buffer_readsample_interpolated(buffer_info_t* buf, int channel, float index, int group);
 void		buffer_writesample(buffer_info_t* buf, int channel, long index, int group, float sample);
+void 		buffer_write_sampleblock(buffer_info_t* buf, long index, long size, int group, float* Lbuf, float* Rbuf);
 float 		buffer_get_sampleratefactor(buffer_info_t* buf, long srate);
 #endif
