@@ -86,22 +86,28 @@ submix_t* submix_new (thread_info_t *info, int id, char* name){
 	/* this is a hack, because we don't need main in L+R */
 		info->outside = 100;
 		snprintf(tmp,MAX_NAMESIZE,"%s_in_L",name);
-		submix->port_inL = jack_port_register (info->jack_info->client, tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
+		submix->port_inL = jack_port_register (info->jack_info->client, 
+					tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
 		snprintf(tmp,MAX_NAMESIZE,"%s_in_R",name);
-		submix->port_inR = jack_port_register (info->jack_info->client, tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
+		submix->port_inR = jack_port_register (info->jack_info->client, 
+					tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
 
 		snprintf(tmp,MAX_NAMESIZE,"%s_in_Position",name);
-		submix->port_inP = jack_port_register (info->jack_info->client, tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
+		submix->port_inP = jack_port_register (info->jack_info->client, 
+					tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
 
 		snprintf(tmp,MAX_NAMESIZE,"%s_out_Position",name);
-		submix->port_outP = jack_port_register (info->jack_info->client, tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+		submix->port_outP = jack_port_register (info->jack_info->client, 
+					tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
 		info->outside = 101;
 
 	}
 	snprintf(tmp,MAX_NAMESIZE,"%s_out_L",name);
-	submix->port_outL = jack_port_register (info->jack_info->client, tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+	submix->port_outL = jack_port_register (info->jack_info->client, 
+				tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
 	snprintf(tmp,MAX_NAMESIZE,"%s_out_R",name);
-	submix->port_outR = jack_port_register (info->jack_info->client, tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+	submix->port_outR = jack_port_register (info->jack_info->client, 
+				tmp, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
 	free(tmp);
 
 	submix->id = id;
@@ -347,7 +353,10 @@ static void* mix_thread(void *arg){
 	int noutports = 0;
 	float mixfactor = .0;
 	const char **ports;
-	float L,R,P;
+/*	float L,R,P;*/
+	float *Lbuf = NULL;
+	float *Rbuf = NULL;
+	float *Pbuf = NULL;
 	double oldrecpos = 0.0;
 	double newrecpos = 0.0;
 	float* mainbufL = (float*) malloc(sizeof(float) * DEFAULT_RB_SIZE);
@@ -397,7 +406,6 @@ static void* mix_thread(void *arg){
 	while (1){
 		if ((info->can_process == 1) || (info->can_process == -1)){
 			info->can_process = 0;
-			j = 0;
 			info->jack_info->load = jack_cpu_load (info->jack_info->client);
 			info->jack_info->freewheeling = info->freewheeling;
 			info->outside = 300;
@@ -432,7 +440,7 @@ static void* mix_thread(void *arg){
 			info->outside = 304;
 
 			memset (mainbufL,0,info->rb_size/*last_nframes*/ * info->sample_size);
-			memset (mainbufR,0,info->rb_size/*iast_nframes*/ * info->sample_size);
+			memset (mainbufR,0,info->rb_size/*last_nframes*/ * info->sample_size);
 
 			info->outside = 305;
 	
@@ -454,10 +462,12 @@ static void* mix_thread(void *arg){
 					/* LATER: move connecting to gui thread / looper.c */
 					if ((ports = jack_get_ports (info->jack_info->client, NULL, NULL, 
 						JackPortIsPhysical|JackPortIsOutput))){
-						if (jack_connect (info->jack_info->client,ports[0], jack_port_name (submix->port_inL))){
+						if (jack_connect (info->jack_info->client,ports[0], 
+									jack_port_name (submix->port_inL))){
 							fprintf (stderr, "cannot connect looper input port\n");
 						}
-						if (jack_connect (info->jack_info->client,ports[1], jack_port_name (submix->port_inR))){
+						if (jack_connect (info->jack_info->client,ports[1], 
+									jack_port_name (submix->port_inR))){
 							fprintf (stderr, "cannot connect looper input port\n");
 						}
 						free (ports);
@@ -473,10 +483,6 @@ static void* mix_thread(void *arg){
 				if (data->buf && data->id){
 					if (data->buf->status == BUFFER_STATUS_READY){
 						info->outside = 310;
-						if (jack_port_connected(submix->port_outL) || jack_port_connected(submix->port_outR)){
-							noutports++;
-							submix->did_output = 1;
-						}else 	submix->did_output = 0;
 						if (jack_port_connected(submix->port_outP))
 							submix->did_pointer = 1;
 						else 	submix->did_pointer = 0;
@@ -485,7 +491,8 @@ static void* mix_thread(void *arg){
 							submix->needs_input = 1;
 						else 	submix->needs_input = 0;
 
-						if ((data->playmode == LOOP_PLAYMODE_EXTERNALPOS) && jack_port_connected(submix->port_inP))
+						if ((data->playmode == LOOP_PLAYMODE_EXTERNALPOS) && 
+								jack_port_connected(submix->port_inP))
 							submix->needs_pointer = 1;
 						else	submix->needs_pointer = 0;
 	
@@ -493,39 +500,60 @@ static void* mix_thread(void *arg){
 			   			jacks buffer_size_callback */
 						/* for now, it's just the last number of samples
 			   			from process() */
+						i = j;
 						j = info->last_nframes;
 						if (jack_ringbuffer_read_space(submix->rb_inL) > j * info->sample_size){
 							j = jack_ringbuffer_read_space(submix->rb_inL) / info->sample_size;
 						}
+						if (i != j){ /* check buffer sizes */
+/*							printf ("n_frames change: %ld -> %ld\n",(long)i, (long)j);*/
+							if (Lbuf){
+								Lbuf = realloc (Lbuf, info->sample_size * j);
+								Rbuf = realloc (Rbuf, info->sample_size * j);
+								Pbuf = realloc (Pbuf, info->sample_size * j);
+							} else {
+								Lbuf = calloc (info->sample_size, j);
+								Rbuf = calloc (info->sample_size, j);
+								Pbuf = calloc (info->sample_size, j);
+							}
+						}
+
 						oldrecpos = looperdata_get_recpos(data);
-						looperdata_lock(data);
-						info->outside = 311;
-						for (i = 0; i < j;i++){
-							/* recording */
-							if (submix->needs_input){
-								jack_ringbuffer_read (submix->rb_inL,(void *) &L, info->sample_size);
-								jack_ringbuffer_read (submix->rb_inR,(void *) &R, info->sample_size);
-								looperdata_write_sample(data, &L, &R);
-							}
+                                                looperdata_lock(data);
+                                                info->outside = 311;
 
-							if (submix->needs_pointer){
-								jack_ringbuffer_read (submix->rb_inP,(void *) &P, info->sample_size);
-								looperdata_set_pointer(data,&P);
-							}
+						if (submix->needs_input){
+							jack_ringbuffer_read (submix->rb_inL,
+								(void *) Lbuf, info->sample_size * j);
+							jack_ringbuffer_read (submix->rb_inR,
+                                                                (void *) Rbuf, info->sample_size * j);
+/*							looperdata_write_sampleblock(data, j, Lbuf, Rbuf);*/
+						}							
+						if (submix->needs_pointer){
+							jack_ringbuffer_read (submix->rb_inP,
+								(void *) Pbuf, info->sample_size * j);
+						}
+					
+						submix->did_output = 0;	
 
-							/* playback */
-							looperdata_calc_sample(data,&L,&R);
-							*(mainbufL + i) += L;
-							*(mainbufR + i) += R;
+						looperdata_calc_sampleblock(data, j, Lbuf, Rbuf, Pbuf);
 
-							if (submix->did_output){
-								jack_ringbuffer_write (submix->rb_outL,(void *)&L,info->sample_size);
-								jack_ringbuffer_write (submix->rb_outR,(void *)&R,info->sample_size);
-							}
-							/* position if needed */
-							if (submix->did_pointer){
-								looperdata_get_pointer(data,&P);
-								jack_ringbuffer_write (submix->rb_outP,(void *)&P,info->sample_size);
+						if (data->isplaying){	
+							if (submix->did_pointer)
+                                                        	jack_ringbuffer_write (submix->rb_outP,
+                                                                	(void *)Pbuf, info->sample_size * j);
+							noutports++;
+							for (i = 0; i < j;i++){
+                                                                *(mainbufL + i) += *(Lbuf + i);
+                                                                *(mainbufR + i) += *(Rbuf + i);         
+                                                        }
+							if (jack_port_connected(submix->port_outL) ||
+                                                        	jack_port_connected(submix->port_outR)){	
+								jack_ringbuffer_write (submix->rb_outL,
+									(void *)Lbuf, info->sample_size * j);
+								jack_ringbuffer_write (submix->rb_outR,
+                                                                	(void *)Rbuf, info->sample_size * j);
+								submix->did_output = 1;
 							}
 						}
 						looperdata_unlock (data);
@@ -541,13 +569,15 @@ static void* mix_thread(void *arg){
 									if (data->buf->normalized){
 										looperdata_lock(data2);
 										looperdata_write_dirtyblock(
-											data2,0.,(double)(buffer_get_size(data2->buf)));
+											data2,0.,(double)(
+												buffer_get_size(data2->buf)));
 										looperdata_unlock(data2);
 									}else{
 										newrecpos = looperdata_get_recpos(data);
 										looperdata_lock(data2);
 										if (newrecpos < oldrecpos){
-											looperdata_write_dirtyblock(data2, oldrecpos,
+											looperdata_write_dirtyblock(data2, 
+												oldrecpos,
 												looperdata_get_recend(data));
 											if (newrecpos > looperdata_get_recstart(data)){
 												looperdata_write_dirtyblock(data2,
@@ -593,7 +623,6 @@ static void* mix_thread(void *arg){
 /*				info->xruns = 0;*/
 			}
 			info->outside = 316;
-/*			gdk_threads_leave(); */
 			/* write main out data */
 			jack_ringbuffer_write (info->submixes[0]->rb_outL,(void *)mainbufL,info->last_nframes * info->sample_size);
 		       	jack_ringbuffer_write (info->submixes[0]->rb_outR,(void *)mainbufR,info->last_nframes * info->sample_size);	
@@ -614,11 +643,7 @@ static void* mix_thread(void *arg){
 
 jack_info_t* mixer_launch (speciallist_t *looperdatalist){
 	thread_info_t *info;
-/*
-	pthread_attr_t attr;
-	int policy;
-	int result;
-*/
+	long waited = 0;
 
 	/* set up communication data */
 	info = (thread_info_t*) malloc(sizeof(thread_info_t));
@@ -663,7 +688,12 @@ jack_info_t* mixer_launch (speciallist_t *looperdatalist){
 /*	pthread_create (&info->mix_thread_id, &attr, mix_thread, info);*/
 	pthread_create (&info->mix_thread_id, NULL, mix_thread, info);
 
-	while (!info->can_process);
+	while (!info->can_process && (++waited < 300)){ 
+		usleep(10);
+	}
+		
+
+	if (info->can_process < 1) return info->jack_info;
 
 	if (jack_activate (info->jack_info->client)) {
                 fprintf (stderr, "cannot activate client\n");
