@@ -33,6 +33,7 @@ typedef struct _looper_data {
 /*	GMutex		*looperdatamutex; */
 	pthread_mutex_t looperdatamutex;
 	buffer_info_t 	*buf;
+	buffer_info_t	*clickmode_buf;
 	int		buffer_group;
 	int		id;
 	long 		samplerate;
@@ -60,7 +61,7 @@ typedef struct _looper_data {
 	float		outmaxR;
 	double		playpos;
 	double		playindex;
-	double		recpos;
+	long		recpos;
 	float		pointer;
 	long		loopstart;
 	long		loopend;
@@ -71,6 +72,7 @@ typedef struct _looper_data {
 	int		linkrec2play;
 	int		grainpitchmode;
 	double		grainpitchbend;
+	double		stereodiversity;
 	int		keepabsolute;
 	int		limiter;
 	float		limit;
@@ -100,6 +102,8 @@ typedef struct _looper_data {
 	double*		grainspeeds;
 	double*		grainpitchtable;
 	double*		grainpitchvolumes;
+	double*		grainpansL;
+	double*		grainpansR;
 	long*		grainoffsets;
 	long*		grainsizes;
 	int*		grainfadelengths;
@@ -118,6 +122,9 @@ void looperdata_unlock(looper_data_t* data);
 
 void looperdata_copy_settings (looper_data_t* dst, looper_data_t* src);
 
+void looperdata_set_clickmode_buf(looper_data_t* data, buffer_info_t* buf);
+buffer_info_t* looperdata_get_clickmode_buf(looper_data_t* data);
+
 void looperdata_write_dirtyblock(looper_data_t* data, double start, double end);
 int looperdata_read_once_dirtyblock(looper_data_t* data, 
 	double* start, double* end);
@@ -151,7 +158,7 @@ void looperdata_set_graindensity(looper_data_t* data,int density);
 void looperdata_set_playmode(looper_data_t* data,int mode);
 void looperdata_set_recmode(looper_data_t* data,int mode);
 void looperdata_set_playpos(looper_data_t* data, double playpos);
-void looperdata_set_recpos(looper_data_t* data, double recpos);
+void looperdata_set_recpos(looper_data_t* data, long recpos);
 void looperdata_set_grainpitchmode (looper_data_t* data, int mode);
 int looperdata_get_grainpitchmode (looper_data_t* data);
 void looperdata_set_grainpitchtablevalue (looper_data_t* data, double value, double vol);
@@ -161,6 +168,8 @@ double looperdata_get_maxgrainspeed(looper_data_t* data);
 long looperdata_get_mingrainlength(looper_data_t* data);
 long looperdata_get_maxgrainlength(looper_data_t* data);
 int looperdata_get_graindensity(looper_data_t* data);
+void looperdata_set_stereodiversity(looper_data_t* data, double diversity);
+double looperdata_get_stereodiversity(looper_data_t* data);
 
 void looperdata_set_reverblength(looper_data_t* data, long length);
 long looperdata_get_reverblength(looper_data_t* data);
@@ -176,7 +185,7 @@ float looperdata_get_limit(looper_data_t* data);
 void looperdata_set_limit(looper_data_t* data, float limit);
 double looperdata_get_speed(looper_data_t* data);
 double looperdata_get_playpos(looper_data_t* data);
-double looperdata_get_recpos(looper_data_t* data);
+long looperdata_get_recpos(looper_data_t* data);
 int looperdata_get_cm_size(looper_data_t* data);
 char* looperdata_cm2str (looper_data_t* data, char* str);
 void looperdata_str2cm (looper_data_t* data, char* str, int cm_size);
@@ -184,8 +193,10 @@ void looperdata_str2cm (looper_data_t* data, char* str, int cm_size);
 void looperdata_get_playmeters(looper_data_t* data,float* L,float* R);
 void looperdata_get_recmeters(looper_data_t* data,float* L,float* R);
 
-void looperdata_write_sample(looper_data_t* data,float* L, float* R);
-void looperdata_calc_sample(looper_data_t* data,float* L, float* R);
+/*void looperdata_write_sample(looper_data_t* data,float* L, float* R);*/
+void looperdata_write_sampleblock(looper_data_t* data, long size, float* Lbuf, float* Rbuf);
+/*void looperdata_calc_sample(looper_data_t* data,float* L, float* R);*/
+void looperdata_calc_sampleblock(looper_data_t* data, long size, float* Lbuf, float* Rbuf, float* Pbuf);
 
 void looperdata_set_pointer(looper_data_t* data, float *P);
 void looperdata_get_pointer(looper_data_t* data, float *P);
