@@ -61,6 +61,7 @@ looper_settings_t *active_settings = NULL;
 const float initial_length = 0.05; /* in seconds (was 0.05)*/
 const float initial_speed = 1;
 const float initial_ngrains = 15;
+const float max_ngrains = 15;
 const float initial_volume  = 0.9;
 const int initial_graindensity = 100;
 const double mintimediff = 20000.;
@@ -149,6 +150,7 @@ void reset_looperdata(looper_data_t *looper_data, jack_info_t *jack_info){
         looperdata_set_recend(looper_data, jack_info->samplerate  * initial_length);
         looperdata_set_mingrainlength(looper_data, jack_info->samplerate  * initial_length);
         looperdata_set_maxgrainlength(looper_data, jack_info->samplerate  * initial_length);
+	looperdata_set_stereodiversity(looper_data, 0.);
 	looperdata_set_playpos (looper_data, 0);
 	looperdata_set_recpos (looper_data, 0);
 	set_distortion(looper_data,0);
@@ -427,14 +429,14 @@ void tranzport_pollall (tranzport_t *tranzport, speciallist_t *looperdatalist, j
 			if (tranzport_mode & TRANZPORT_BUTTON_TRACKREC){
 				tranzport_mode ^= TRANZPORT_BUTTON_TRACKREC;
 				tranzport_set_led(tranzport, TRANZPORT_LIGHT_TRACKREC,0);
-				set_lcd (tranzport,"GrainPitchButton off",0,"                    ",0,20);
+				set_lcd (tranzport,"SpreadStereoBtn off ",0,"                    ",0,20);
 			}else{
 				float f;
 				tranzport_mode |= TRANZPORT_BUTTON_TRACKREC;
 				tranzport_set_led(tranzport, TRANZPORT_LIGHT_TRACKREC,1);
 				wld = speciallist_get_first(looperdatalist);
-				f = looperdata_get_mingrainspeed(wld);
-				set_lcd_value_float (tranzport,"GrainPitch",f,0.,100.);
+				f = looperdata_get_stereodiversity(wld);
+				set_lcd_value_float (tranzport,"SpreadStereo",f,0.,1.);
 			}
 		} /* TRACKREC BUTTON == PITCH SHIFT PRESSED */
 
@@ -495,7 +497,7 @@ void tranzport_pollall (tranzport_t *tranzport, speciallist_t *looperdatalist, j
                                 tranzport_set_led(tranzport, TRANZPORT_LIGHT_LOOP,1);
 				wld = speciallist_get_first(looperdatalist);
                                 d = looperdata_get_ngrains(wld);
-				set_lcd_value (tranzport,"Nr. of Grains",d,1,15);
+				set_lcd_value (tranzport,"Nr. of Grains",d,1,max_ngrains);
                         }
                 } /* LOOP BUTTON == NRofGRAINS PRESSED */
 
@@ -544,6 +546,53 @@ void tranzport_pollall (tranzport_t *tranzport, speciallist_t *looperdatalist, j
 			snprintf (msg,21,"preset: %s           ",active_settings->name);
                         set_lcd (tranzport,msg,0,NULL,0,20);
 		}
+		if (tranzport_pressed(tranzport, TRANZPORT_BUTTON_REWIND)){
+			float f;
+                        wld = speciallist_get_first(looperdatalist);
+                        f = looperdata_get_mingrainspeed(wld);
+                        f *= pow(semitone,-1.);
+                        if (tranzport_buttonstate(tranzport, TRANZPORT_BUTTON_SHIFT))
+                                f *= pow(semitone,-1.);
+                        if ((f > 0.0001) && (f < 100.)){
+                                while (wld){    /* grain speed */
+                                        looperdata_lock(wld);
+                                        looperdata_set_mingrainspeed(wld,f);
+                                        looperdata_set_maxgrainspeed(wld,f);
+                                        looperdata_unlock(wld);
+                                        wld = speciallist_get_next(looperdatalist,wld);
+                                }
+                                set_lcd_value_float (tranzport,"GrainPitch",f,0.,100.);
+                        }
+                }
+		if (tranzport_pressed(tranzport, TRANZPORT_BUTTON_FASTFORWARD)){
+			float f;
+                        wld = speciallist_get_first(looperdatalist);
+                        f = looperdata_get_mingrainspeed(wld);
+                        f *= pow(semitone,1.);
+                        if (tranzport_buttonstate(tranzport, TRANZPORT_BUTTON_SHIFT))
+                                f *= pow(semitone,1.);
+                        if ((f > 0.0001) && (f < 100.)){
+                                while (wld){    /* grain speed */
+                                        looperdata_lock(wld);
+                                        looperdata_set_mingrainspeed(wld,f);
+                                        looperdata_set_maxgrainspeed(wld,f);
+                                        looperdata_unlock(wld);
+                                        wld = speciallist_get_next(looperdatalist,wld);
+                                }
+                                set_lcd_value_float (tranzport,"GrainPitch",f,0.,100.);
+                        }
+                }
+		if (tranzport_pressed(tranzport, TRANZPORT_BUTTON_STOP)){
+			while (wld){    /* grain speed */
+				looperdata_lock(wld);
+                                looperdata_set_mingrainspeed(wld,1.);
+                                looperdata_set_maxgrainspeed(wld,1.);
+                                looperdata_unlock(wld);
+                                wld = speciallist_get_next(looperdatalist,wld);
+                        }
+			set_lcd_value_float (tranzport,"GrainPitch",1.,0.,100.);
+		}
+		
 
 		if (tranzport_pressed(tranzport, TRANZPORT_BUTTON_ADD)){
 			char msg[21] = "                     ";
@@ -565,10 +614,11 @@ void tranzport_pollall (tranzport_t *tranzport, speciallist_t *looperdatalist, j
 
 		/* ***************** RESET *************************/
 		if (tranzport_pressed(tranzport, TRANZPORT_BUTTON_UNDO)){
+/*
 			if (tranzport_mode & TRANZPORT_BUTTON_TRACKREC){
 				set_lcd_value_float (tranzport,"GrainPitch",1.,0.,100.);
                                 wld = speciallist_get_first(looperdatalist);
-                                while (wld){    /* grain speed */
+                                while (wld){  
                                         looperdata_lock(wld);
                                         looperdata_set_mingrainspeed(wld,1.);
                                         looperdata_set_maxgrainspeed(wld,1.);
@@ -576,6 +626,20 @@ void tranzport_pollall (tranzport_t *tranzport, speciallist_t *looperdatalist, j
                                         wld = speciallist_get_next(looperdatalist,wld);
                                 }
                         }
+*/
+
+			if (tranzport_mode & TRANZPORT_BUTTON_TRACKREC){
+                                set_lcd_value_float (tranzport,"SpreadStereo",0.,0.,1.);
+                                wld = speciallist_get_first(looperdatalist);
+                                while (wld){  
+                                        looperdata_lock(wld);
+                                        looperdata_set_stereodiversity(wld,0.);
+                                        looperdata_unlock(wld);
+                                        wld = speciallist_get_next(looperdatalist,wld);
+                                }
+                        }
+
+			
 			if (tranzport_mode & TRANZPORT_BUTTON_TRACKMUTE){
 				long d = jack_info->samplerate * initial_length;
                                 set_lcd_value (tranzport,"LoopLength",d ,0, jack_info->samplerate * 60);
@@ -599,11 +663,11 @@ void tranzport_pollall (tranzport_t *tranzport, speciallist_t *looperdatalist, j
                                 }
                         }
 			if (tranzport_mode & TRANZPORT_BUTTON_LOOP){
-				set_lcd_value (tranzport,"Nr. of Grains",15,1,15);
+				set_lcd_value (tranzport,"Nr. of Grains",max_ngrains,1,max_ngrains);
 				wld = speciallist_get_first(looperdatalist);
                                 while (wld){    /* grain speed */
                                         looperdata_lock(wld);
-					looperdata_set_ngrains(wld,15);
+					looperdata_set_ngrains(wld,max_ngrains);
 					looperdata_unlock(wld);
                                         wld = speciallist_get_next(looperdatalist,wld);
 				}
@@ -649,13 +713,13 @@ void tranzport_pollall (tranzport_t *tranzport, speciallist_t *looperdatalist, j
 	
 		/* ***************************   SHUTTLE WHEEL ************************** */
 		if ((wheel = tranzport_get_wheel(tranzport))){
-			if (tranzport_mode & TRANZPORT_BUTTON_TRACKREC){			
+			if (!tranzport_mode){
 				float f;
 				wld = speciallist_get_first(looperdatalist);
 				f = looperdata_get_mingrainspeed(wld);
-				f *= pow(semitone,(double)wheel);
 				if (tranzport_buttonstate(tranzport, TRANZPORT_BUTTON_SHIFT))
-					f *= pow(semitone,(double)wheel);
+					f *= 1 + ((double)wheel * .1);
+				else f *= 1 + ((double)wheel * .01);
 				if ((f > 0.0001) && (f < 100.)){
                                		while (wld){    /* grain speed */
                                         	looperdata_lock(wld);
@@ -666,7 +730,25 @@ void tranzport_pollall (tranzport_t *tranzport, speciallist_t *looperdatalist, j
                                 	}
 					set_lcd_value_float (tranzport,"GrainPitch",f,0.,100.);
 				}
-			} /* end pitch shift == TRACKREC */
+			} /* end pitch shift == NO BUTTON PRESSED */
+
+			if (tranzport_mode & TRANZPORT_BUTTON_TRACKREC){
+				float f;
+				wld = speciallist_get_first(looperdatalist);
+				f = looperdata_get_stereodiversity(wld);
+				f += (double)wheel * .05;
+				if (tranzport_buttonstate(tranzport, TRANZPORT_BUTTON_SHIFT))
+					f += (double)wheel * .05;
+				if (f < 0.) f = 0.;
+				if (f > 1.) f = 1.;
+				while (wld){
+					looperdata_lock(wld);
+					looperdata_set_stereodiversity(wld,f);
+					looperdata_unlock(wld);
+                                        wld = speciallist_get_next(looperdatalist,wld);
+				}
+				set_lcd_value_float (tranzport,"SpreadStereo",f,0.,1.);
+			}/* end stereo diversity = TRACKREC */
 
 			if (tranzport_mode & TRANZPORT_BUTTON_TRACKMUTE){
 				long d = 0;
@@ -712,14 +794,14 @@ void tranzport_pollall (tranzport_t *tranzport, speciallist_t *looperdatalist, j
                                 	d = looperdata_get_ngrains(wld);
                                         d += wheel;
                                         if (d < 1) d = 1;
-                                        if (d > 15) d = 15; 
+                                        if (d > max_ngrains) d = max_ngrains; 
                                         looperdata_lock(wld);
 /*                                      printf ("ngrains:%d\n", diff);*/
                                         looperdata_set_ngrains(wld, d);
                                         looperdata_unlock(wld);
                                         wld = speciallist_get_next(looperdatalist,wld);
                                 }	
-				set_lcd_value (tranzport,"Nr. of Grains",d,1,15);
+				set_lcd_value (tranzport,"Nr. of Grains",d,1,max_ngrains);
 			}
 
                      	if (tranzport_mode & TRANZPORT_BUTTON_PUNCH){
@@ -970,7 +1052,7 @@ void shuttle_pollall (shuttle_data_t  *shuttle, speciallist_t *looperdatalist, j
 						int d = looperdata_get_ngrains(wld);
 						int diff = d + (shuttle->shuttledelta / 4);
 						if (diff < 1) diff = 1;
-						if (diff > 15) diff = 15;
+						if (diff > max_ngrains) diff = max_ngrains;
 						if ((diff > 0) && (diff < 16)){
 							looperdata_lock(wld);
 /*							printf ("ngrains:%d\n", diff);*/
@@ -1133,6 +1215,7 @@ void create_settings(buffer_info_t* buf, jack_info_t* jack_info){
         looperdata_set_grainpitchtablevalue (ld, note, 1.0);
         note = midi2pitch(56);
         looperdata_set_grainpitchtablevalue (ld, note, 1.0);
+	looperdata_set_stereodiversity(ld,0.);
 	ld->isrecording = 1;
         looperdata_unlock(ld);
         ls = malloc(sizeof(looper_settings_t));
@@ -1157,6 +1240,7 @@ void create_settings(buffer_info_t* buf, jack_info_t* jack_info){
         looperdata_set_loopend (ld, 256);
         looperdata_set_recend (ld, 256);
         looperdata_set_mingrainlength(ld,256);
+	looperdata_set_stereodiversity(ld,0.);
 	ld->isrecording = 1;
         looperdata_unlock(ld);
         set_distortion(ld,1.);  
@@ -1173,6 +1257,7 @@ void create_settings(buffer_info_t* buf, jack_info_t* jack_info){
         looperdata_set_loopend (ld, 13);
         looperdata_set_recend (ld, 13);
         looperdata_set_mingrainlength(ld,13);
+	looperdata_set_stereodiversity(ld,0.);
         ld->isrecording = 1;
         looperdata_unlock(ld);
         ls = malloc(sizeof(looper_settings_t));
@@ -1191,6 +1276,7 @@ void create_settings(buffer_info_t* buf, jack_info_t* jack_info){
         looperdata_set_loopend (ld, 79);
         looperdata_set_recend (ld, 79);
         looperdata_set_mingrainlength(ld,79);
+	looperdata_set_stereodiversity(ld,0.);
         ld->isrecording = 1;
         looperdata_unlock(ld);
         ls = malloc(sizeof(looper_settings_t));
@@ -1204,6 +1290,7 @@ void create_settings(buffer_info_t* buf, jack_info_t* jack_info){
         looperdata_lock(ld);
         looperdata_set_mingrainspeed(ld,2.);
         looperdata_set_maxgrainspeed(ld,2.);
+	looperdata_set_stereodiversity(ld,0.);
 	ld->isrecording = 1;
         looperdata_unlock(ld);
         set_distortion(ld,1.);
@@ -1218,6 +1305,7 @@ void create_settings(buffer_info_t* buf, jack_info_t* jack_info){
         looperdata_lock(ld);
 	looperdata_set_mingrainspeed(ld,.5);
         looperdata_set_maxgrainspeed(ld,.5);
+	looperdata_set_stereodiversity(ld,0.);
 	ld->isrecording = 1;
         looperdata_unlock(ld);
         set_distortion(ld,1.);
@@ -1232,6 +1320,7 @@ void create_settings(buffer_info_t* buf, jack_info_t* jack_info){
         looperdata_lock(ld);
         looperdata_set_mingrainspeed(ld,.25);
         looperdata_set_maxgrainspeed(ld,.25);
+	looperdata_set_stereodiversity(ld,0.);
 	ld->isrecording = 1;
         looperdata_unlock(ld);
         set_distortion(ld,1.);
@@ -1246,6 +1335,7 @@ void create_settings(buffer_info_t* buf, jack_info_t* jack_info){
         looperdata_lock(ld);
         looperdata_set_mingrainspeed(ld,.05);
         looperdata_set_maxgrainspeed(ld,.05);
+	looperdata_set_stereodiversity(ld,0.);
 	ld->isrecording = 1;
         looperdata_unlock(ld);
         set_distortion(ld,1.);
