@@ -1,3 +1,4 @@
+#include <locale.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -21,11 +22,14 @@ int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath,
 	int err;
 	char txt[128];
 	char *cmd = NULL;
+	char *old_locale;
 	GList *listptr = looperlist;
 	looper_gui_t *looper = NULL;
 	buffer_info_t *buf = NULL;
 	xmlNodePtr topptr, looperptr, bufferptr;
 
+	old_locale = setlocale(LC_NUMERIC,"C");
+
 	doc = xmlNewDoc((xmlChar*)XML_DEFAULT_VERSION);
 	topptr = xmlNewDocNode(doc, NULL, (xmlChar*)"kluppe", NULL);
 	xmlAddChild ((xmlNodePtr)doc, topptr);
@@ -46,6 +50,8 @@ int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath,
 			looperptr  =  xmlNewChild (topptr, NULL, (xmlChar*)"looper", NULL);
 			sprintf (txt, "%d", looper->id);
 			xmlNewChild (looperptr, NULL, (xmlChar*)"id", (xmlChar*)txt);	
+			sprintf (txt, "%d", looper_get_position(looper));
+                        xmlNewChild (looperptr, NULL, (xmlChar*)"position",(xmlChar*)txt);
 			
 			if (looper->data){
 				/* BUF??? */
@@ -67,7 +73,7 @@ int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath,
 
 				sprintf (txt, "%f", looper->data->playpos);
 				xmlNewChild (looperptr, NULL, (xmlChar*)"playpos",(xmlChar*)txt);
-				sprintf (txt, "%fd", looper->data->recpos);
+				sprintf (txt, "%ld", looper->data->recpos);
                                 xmlNewChild (looperptr, NULL, (xmlChar*)"recpos",(xmlChar*)txt);
 				sprintf (txt, "%ld", looper->data->loopstart);
                                 xmlNewChild (looperptr, NULL, (xmlChar*)"loopstart",(xmlChar*)txt);
@@ -129,6 +135,8 @@ int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath,
 				xmlNewChild (looperptr, NULL, (xmlChar*)"ngrains",(xmlChar*)txt);
 				sprintf (txt, "%d", looper->data->graindensity);
 				xmlNewChild (looperptr, NULL, (xmlChar*)"graindensity",(xmlChar*)txt);
+				sprintf (txt, "%f", looper->data->stereodiversity);
+				xmlNewChild (looperptr, NULL, (xmlChar*)"stereodiversity",(xmlChar*)txt);
 				sprintf (txt, "%f", looper->data->mingrainspeed);
 				xmlNewChild (looperptr, NULL, (xmlChar*)"mingrainspeed",(xmlChar*)txt);
 				sprintf (txt, "%f", looper->data->maxgrainspeed);
@@ -139,6 +147,11 @@ int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath,
                                 xmlNewChild (looperptr, NULL, (xmlChar*)"maxgrainlength",(xmlChar*)txt);
 				sprintf (txt, "%d", looperdata_get_grainpitchmode(looper->data));
 				xmlNewChild (looperptr, NULL, (xmlChar*)"grainpitchmode",(xmlChar*)txt);
+
+				sprintf (txt, "%s", buffer_get_shortname(
+					looperdata_get_clickmode_buf(looper->data)));
+
+				xmlNewChild (looperptr, NULL, (xmlChar*)"clickmode",(xmlChar*)txt);
 				/* grainpitchbend??? */
 				sprintf (txt, "%d", looperdata_get_midichannel(looper->data));
 				xmlNewChild (looperptr, NULL, (xmlChar*)"midichannel",(xmlChar*)txt);
@@ -176,6 +189,8 @@ int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath,
 		xmlNewChild (bufferptr, NULL, (xmlChar*)"channels",(xmlChar*)txt);
 		sprintf (txt, "%ld", buffer_get_size(buf));
 		xmlNewChild (bufferptr, NULL, (xmlChar*)"size",(xmlChar*)txt);
+		sprintf (txt, "%lf", buffer_get_volume_factor(buf));
+                xmlNewChild (bufferptr, NULL, (xmlChar*)"volumefactor",(xmlChar*)txt);
 		buf = speciallist_get_next(bufferlist,buf);
 	}
 
@@ -183,6 +198,8 @@ int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath,
 		xmlFreeDoc (doc);
 		return -1;
 	}
+
+	setlocale(LC_NUMERIC,old_locale);
 		
 	return 0;
 }
@@ -222,52 +239,76 @@ void close_config_reader (configfile_handler_t* handler){
 static int subnode2int (xmlNodePtr parent, const char *key){
 	xmlNodePtr node;
 	xmlChar *content;
+	int i;
+	char * old_locale = setlocale(LC_NUMERIC,"C");
 
 	for (node = parent->children; node; node = node->next){
 		if (!strcmp((char*)node->name, (char*)key)){
 			content = xmlNodeGetContent (node);
-			return atoi ((char*)content);
+			i = atoi ((char*)content);
+			setlocale(LC_NUMERIC, old_locale);
+			return i;
 		}
 	}
+	
+	setlocale(LC_NUMERIC, old_locale);
 	return 0;
 }
 
 static long subnode2long (xmlNodePtr parent, const char *key){
         xmlNodePtr node;
         xmlChar *content;
+	long l;
+	char * old_locale = setlocale(LC_NUMERIC,"C");
 
         for (node = parent->children; node; node = node->next){
                 if (!strcmp((char*)node->name, (char*)key)){
                         content = xmlNodeGetContent (node);
-                        return atol ((char*)content);
+			l = atol ((char*)content);
+			setlocale(LC_NUMERIC, old_locale);
+                        return l;
                 }
         }
+
+	setlocale(LC_NUMERIC, old_locale);
 	return 0;
 }
 
 static float subnode2float (xmlNodePtr parent, const char *key){
         xmlNodePtr node;
         xmlChar *content;
+	float f;
+	char * old_locale = setlocale(LC_NUMERIC,"C");
 
         for (node = parent->children; node; node = node->next){
                 if (!strcmp((char*)node->name, (char*)key)){
                         content = xmlNodeGetContent (node);
-                        return (float)atof ((char*)content);
+			f = (float)atof ((char*)content);
+			setlocale(LC_NUMERIC, old_locale);
+                        return f;
                 }
         }
+
+	setlocale(LC_NUMERIC, old_locale);
 	return 0.0;
 }
 
 static double subnode2double (xmlNodePtr parent, const char *key){
         xmlNodePtr node;
         xmlChar *content;
+	double d;
+	char * old_locale = setlocale(LC_NUMERIC,"C");
 
         for (node = parent->children; node; node = node->next){
                 if (!strcmp((char*)node->name,(char*) key)){
                         content = xmlNodeGetContent (node);
-                        return atof ((char*)content);
+			d = atof ((char*)content);
+			setlocale(LC_NUMERIC, old_locale);
+                        return d;
                 }
         }
+
+	setlocale(LC_NUMERIC, old_locale);
 	return 0.0;
 }
 
@@ -285,13 +326,10 @@ static char* subnode2string (xmlNodePtr parent, const char *key){
 }
 
 void config_get_geometry(configfile_handler_t* handler,gint* x, gint* y, gint* width,gint* height){
-/*        xmlNodePtr node;  */
-
 	*x= subnode2int (handler->kluppe,"window-xpos");
 	*y = subnode2int (handler->kluppe,"window-ypos");
         *width = subnode2int (handler->kluppe,"window-width");
         *height = subnode2int (handler->kluppe,"window-height");
-
         return;
 }                       
 
@@ -314,9 +352,23 @@ int config_next_looper (configfile_handler_t* handler){
         return 0;
 }
 
+int config_get_looper_minimized (configfile_handler_t* handler, int id){
+	xmlNodePtr node;
+        xmlNodePtr lptr = NULL; 
+                                
+        for (node = handler->kluppe->children; node; node = node->next){
+                if (!strcmp((char*)node->name, (char*)"looper"))
+                        if (subnode2int(node, "id") == id) lptr = node;
+        }                       
+                        
+        if (lptr) return subnode2int(lptr,"minimized");
+	return 0;
+}
+
 int config_read_looper (configfile_handler_t* handler, looper_gui_t *looper){
 	xmlNodePtr node;
 	xmlNodePtr lptr = NULL;
+	buffer_info_t *buf;
 
 	for (node = handler->kluppe->children; node; node = node->next){
 		if (!strcmp((char*)node->name, (char*)"looper"))
@@ -326,8 +378,13 @@ int config_read_looper (configfile_handler_t* handler, looper_gui_t *looper){
 
 	if (looper &&  lptr){
 		if (looper->data && subnode2int(lptr,"buffer")){
-			looper_set_buffer_by_id(looper,
-                                subnode2int(lptr,"buffer"));
+                        gtk_toggle_button_set_active((GtkToggleButton*)looper->minimizebutton,
+                                (subnode2int(lptr,"minimized") ? TRUE : FALSE));
+
+			looper_set_buffer_by_id(looper,subnode2int(lptr,"buffer"));
+
+			if (subnode2string(lptr,"clickmode"))
+				looper_set_position(looper,subnode2int(lptr,"position"));
 
 			looperdata_set_vol(looper->data,
 				subnode2float(lptr,"vol"));	
@@ -352,12 +409,14 @@ int config_read_looper (configfile_handler_t* handler, looper_gui_t *looper){
 			looperdata_set_playpos(looper->data,
                                 subnode2double(lptr,"playpos"));
                         looperdata_set_recpos(looper->data,
-                                subnode2double(lptr,"recpos"));
+                                subnode2long(lptr,"recpos"));
 
 			looperdata_set_ngrains(looper->data,
                                 subnode2int(lptr,"ngrains"));
 			looperdata_set_graindensity(looper->data,
                                 subnode2int(lptr,"graindensity"));
+			looperdata_set_stereodiversity(looper->data,
+                                subnode2double(lptr,"stereodiversity"));
 
 			looperdata_set_mingrainspeed(looper->data,
                                 subnode2double(lptr,"mingrainspeed"));
@@ -405,9 +464,6 @@ int config_read_looper (configfile_handler_t* handler, looper_gui_t *looper){
                         gtk_toggle_button_set_active((GtkToggleButton*)looper->playbutton,
                                 (subnode2int(lptr,"isplaying") ? TRUE : FALSE));
 
-                        gtk_toggle_button_set_active((GtkToggleButton*)looper->minimizebutton,
-                                (subnode2int(lptr,"minimized") ? TRUE : FALSE));
-
 			gtk_loopview_set_bpm(GTK_LOOPVIEW(looper->loopview),
 				subnode2double(lptr,"bpm"));
 			gtk_loopview_set_bpb(GTK_LOOPVIEW(looper->loopview),
@@ -415,6 +471,19 @@ int config_read_looper (configfile_handler_t* handler, looper_gui_t *looper){
 			gtk_loopview_set_gridoffset(GTK_LOOPVIEW(looper->loopview),
 				subnode2long(lptr,"gridoffset"));
 
+			if (subnode2string(lptr,"clickmode")){ 
+				/* old configfiles have no clickmode*/
+				buf = speciallist_get_first(looper->clickmodes);
+/*				printf ("clickmode buf #1: %p, %p\n",buf, looper->clickmodes);*/
+				while (buf){
+					if (!strcmp(buffer_get_shortname(buf),
+						subnode2string(lptr,"clickmode"))){
+						looperdata_set_clickmode_buf(looper->data, buf);
+					}
+					buf = speciallist_get_next(looper->clickmodes,buf);
+				}
+			}
+
 
 			switch (subnode2int(lptr,"playmode")){
 				case LOOP_PLAYMODE_LOOP:
@@ -490,6 +559,7 @@ int config_read_buffer (configfile_handler_t* handler, buffer_info_t* buf){
 	int file_ok = 0;
 	char *filename;
 	struct stat fstat;
+	float	vol = 0.;
 
 	if (buf && handler->bufferptr){
 		type = subnode2int (handler->bufferptr,"type");
@@ -504,15 +574,17 @@ int config_read_buffer (configfile_handler_t* handler, buffer_info_t* buf){
 		 	if ((type == BUFFER_TYPE_FILEBUFFER) || ((type == 0) && (file_ok == 1))){
 				buffer_loadfile(buf,filename,handler->samplerate);
 			}else{
-				buffer_set_channels(buf, 
-					subnode2int(handler->bufferptr,"channels"));
 				buffer_resize(buf,
 					subnode2long(handler->bufferptr,"size"));
 				buffer_set_shortname(buf,
                                		subnode2string(handler->bufferptr,"shortname"));
+				buffer_set_channels(buf,
+                                        subnode2int(handler->bufferptr,"channels"));
 				buf->pending = BUFFER_STATUS_READY;
 			}
 		}
+		vol = subnode2double(handler->bufferptr,"volumefactor");
+		if(vol) buffer_set_volume_factor(buf,vol);
 	}
 	return 0;
 }
