@@ -160,6 +160,7 @@ static void gtk_loopview_init (GtkLoopview *loopview) {
   	loopview->button = 0;
   	loopview->policy = GTK_UPDATE_CONTINUOUS;
   	loopview->timer = 0;
+	loopview->suspended = FALSE;
 	loopview->peakview = NULL;
   	loopview->looperdata = looperdata_new(NULL,44100, 0);
 /*	loopview->buffer_group = 0;*/
@@ -251,6 +252,7 @@ static void gtk_loopview_destroy (GtkObject *object)
   	g_return_if_fail (GTK_IS_LOOPVIEW (object));
 
   	loopview = GTK_LOOPVIEW (object);
+	if (loopview->peakview) peakview_delete(loopview->peakview);
 
   	if (GTK_OBJECT_CLASS (parent_class)->destroy)
     		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
@@ -274,6 +276,28 @@ void gtk_loopview_set_update_policy (GtkLoopview      *loopview,
   	loopview->policy = policy;
 }
 
+void gtk_loopview_suspend_gui (GtkLoopview *loopview, int suspend){
+	g_return_if_fail (loopview != NULL);
+        g_return_if_fail (GTK_IS_LOOPVIEW (loopview));
+
+	if (suspend){
+		loopview->suspended = TRUE;
+		if (loopview->peakview) peakview_clear(loopview->peakview, 0, 0);
+	} else {
+		loopview->suspended = FALSE;
+		if (loopview->looperdata)
+			if (loopview->looperdata->buf){
+                        	if (!loopview->peakview)
+					loopview->peakview = peakview_new(loopview->looperdata->buf);
+
+				peakview_set_area(loopview->peakview,loopview->zoomstart,
+                                                	loopview->zoomend, loopview->npeaks);
+				peakview_calculate(loopview->peakview,loopview->zoomstart, 
+						loopview->zoomend);
+			}
+	}
+}
+
 void gtk_loopview_set_looperdata (GtkLoopview *loopview, looper_data_t *looperdata){
   	g_return_if_fail (loopview != NULL);
   	g_return_if_fail (GTK_IS_LOOPVIEW (loopview));
@@ -281,7 +305,12 @@ void gtk_loopview_set_looperdata (GtkLoopview *loopview, looper_data_t *looperda
   	loopview->looperdata = looperdata;
 	loopview->zoomstart = 0;
 	loopview->zoomend = 0;
-/*	if (loopview->peakview) peakview_delete(loopview->peakview);*/
+/*
+	if (loopview->peakview) {
+		peakview_delete(loopview->peakview);
+		loopview->peakview = NULL;
+	}
+*/
 	if (loopview->looperdata->buf){
 		loopview->zoomend = buffer_get_size(loopview->looperdata->buf);
 		if (!loopview->peakview)
@@ -290,6 +319,7 @@ void gtk_loopview_set_looperdata (GtkLoopview *loopview, looper_data_t *looperda
 		peakview_set_area(loopview->peakview,loopview->zoomstart, 
 						loopview->zoomend, loopview->npeaks);
 	}else{
+		peakview_set_buffer(loopview->peakview, NULL);
 		gtk_loopview_update_area(loopview,0,loopview->npeaks);
 	}
 	gtk_loopview_set_bpm(loopview,120);
@@ -303,7 +333,7 @@ void gtk_loopview_update_waveform (GtkLoopview *loopview){
 	g_return_if_fail (loopview != NULL);
 	g_return_if_fail (GTK_IS_LOOPVIEW (loopview));
 
-	if (loopview->looperdata->buf)
+	if (loopview->looperdata->buf && !loopview->suspended)
 		peakview_calculate(loopview->peakview,0, loopview->zoomend);
 	else gtk_loopview_update_area(loopview,0,loopview->npeaks);
 /*	gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
@@ -393,7 +423,8 @@ void gtk_loopview_zoom_to_playselection (GtkLoopview *loopview){
 		loopview->grecend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recend);
 /*		gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
 		peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, loopview->npeaks);
-		peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
+		if (!loopview->suspended)
+			peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
 	}
 }
 
@@ -407,7 +438,8 @@ void gtk_loopview_zoom_to_recselection (GtkLoopview *loopview){
 		loopview->grecend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recend);
 /*		gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
 		peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, loopview->npeaks);
-                peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
+		if (!loopview->suspended)
+                	peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
 	}
 }
 
@@ -424,7 +456,8 @@ void gtk_loopview_zoom_to_overview (GtkLoopview *loopview){
 		loopview->grecend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recend);
 /*		gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
 		peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, loopview->npeaks);
-                peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
+		if (!loopview->suspended)
+                	peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
 	}
 }
 
@@ -457,7 +490,8 @@ void gtk_loopview_set_zoom (GtkLoopview *loopview, gdouble zoomstart, gdouble zo
 				loopview->grecend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recend);
 /*				gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
 				peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, loopview->npeaks);
-                		peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
+				if (!loopview->suspended)
+                			peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
 			}
 		}
 	}
@@ -603,21 +637,26 @@ void gtk_loopview_update_cursorposition (GtkLoopview *loopview){
 	if (loopview->looperdata){
 		if (loopview->looperdata->buf){
 			gdk_window_get_pointer (GTK_WIDGET(loopview)->window, &x, &y, &mods);
-			looperdata_lock(loopview->looperdata);
-			while (looperdata_read_once_dirtyblock(loopview->looperdata, &start, &end))
-				peakview_calculate(loopview->peakview, (long)start, (long)end);
-			looperdata_unlock(loopview->looperdata);
+
+//			looperdata_lock(loopview->looperdata);
+			while (looperdata_read_once_dirtyblock(loopview->looperdata, &start, &end)){
+				if (!loopview->suspended){
+					peakview_calculate(loopview->peakview, (long)start, (long)end);
+				}
+			}
+//			looperdata_unlock(loopview->looperdata);
 
 			if (loopview->peakview){
 				peakview_lock(loopview->peakview);
 				while ((pd = peakview_get_next(loopview->peakview))){
 					gtk_loopview_copy_peaks(loopview,pd);
-					peakdata_set_status(pd, PD_STATUS_DEAD);
 				}
 				peakview_unlock(loopview->peakview);
 			}
-			gtk_loopview_set_play_pos (loopview,loopview->looperdata->playpos);
-			gtk_loopview_set_rec_pos (loopview,loopview->looperdata->recpos);
+			gtk_loopview_set_play_pos (loopview,
+				looperdata_get_playpos(loopview->looperdata));
+			gtk_loopview_set_rec_pos (loopview,
+				(double)looperdata_get_recpos(loopview->looperdata));
 		}
 	}
 }
@@ -695,8 +734,7 @@ static void gtk_loopview_size_allocate (GtkWidget     *widget,
 		loopview->pixmap = gdk_pixmap_new(widget->window,allocation->width,allocation->height,-1);
 		
 
-		if ((loopview->width != allocation->width) || (loopview->height != allocation->height)){	
-
+		if (loopview->width != allocation->width){	
 			if (loopview->npeaks){
 				loopview->gloopstart = loopview->gloopstart * allocation->width / loopview->npeaks;
 				loopview->gloopend   = loopview->gloopend   * allocation->width / loopview->npeaks;
@@ -717,11 +755,15 @@ static void gtk_loopview_size_allocate (GtkWidget     *widget,
 			loopview->width = allocation->width;
 			loopview->height = allocation->height;
 /*			gtk_loopview_calc_peaks(loopview,0,allocation->width);*/
+
 			peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, allocation->width);
-			peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
-			if (!loopview->peakview)
-				gtk_loopview_update_area(loopview,0,allocation->width);
-		}else{
+			if (!loopview->suspended)
+				peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
+			gtk_loopview_update_area(loopview,0,allocation->width);
+		} else if (loopview->height != allocation->height){
+			loopview->height = allocation->height;
+			gtk_loopview_update_area(loopview,0,allocation->width);
+		} else{
 			gtk_loopview_update_area(loopview,0,allocation->width);
 		}
     	}/*else printf("not realized\n");*/
@@ -969,17 +1011,23 @@ void gtk_loopview_calc_peaks(GtkLoopview *loopview,guint32 startx, guint32 endx)
 
 void gtk_loopview_copy_peaks(GtkLoopview *loopview, peakdata_t *pd){
 	long i;
-
+	long len = pd->pend - pd->pstart;
 /*	printf ("copy: %p. done:%d\n",pd,pd->done);*/
-	/* LATER: use mmap */
-	for (i = 0; i < (pd->pend - pd->pstart);i++){
-		*(loopview->Lmin + i + pd->pstart) = (gint8)*(pd->Lmin + i);
-		*(loopview->Lmax + i + pd->pstart) = (gint8)*(pd->Lmax + i);
-		if (buffer_get_channels(loopview->looperdata->buf) > 1){
+/*	printf ("copy: pstart:%ld, pend:%ld\n", pd->pstart, pd->pend);*/
+	if (buffer_get_channels(loopview->looperdata->buf) > 1){
+		for (i = 0; i < len;i++){
+			*(loopview->Lmin + i + pd->pstart) = (gint8)*(pd->Lmin + i);
+			*(loopview->Lmax + i + pd->pstart) = (gint8)*(pd->Lmax + i);
 			*(loopview->Rmin + i + pd->pstart) = (gint8)*(pd->Rmin + i);
                 	*(loopview->Rmax + i + pd->pstart) = (gint8)*(pd->Rmax + i);
 		}
+	} else {
+		for (i = 0; i < len;i++){
+			*(loopview->Lmin + i + pd->pstart) = (gint8)*(pd->Lmin + i);
+                        *(loopview->Lmax + i + pd->pstart) = (gint8)*(pd->Lmax + i);	
+		}
 	}
+	peakdata_set_status(pd,PD_STATUS_DEAD);
 /*	printf ("updated:...%ld - %ld\n",pd->pstart, (pd->pstart + i));*/
 	gtk_loopview_update_area(loopview, pd->pstart, pd->pend);
 }
@@ -1136,15 +1184,15 @@ static gboolean gtk_loopview_button_press( GtkWidget *widget,
 		sdiff = loopview->gloopstart - event->x;
 		ediff = event->x - loopview->gloopend;
 		bdiff = loopview->gloopend - loopview->gloopstart;
-		if (((sdiff < maxd) && (sdiff > 0)) || 
-			((sdiff < 0) && (sdiff  > (maxd * -1) 
+		if (((sdiff < maxd) && (sdiff >= 0)) || 
+			((sdiff <= 0) && (sdiff  > (maxd * -1) 
 				&& (sdiff <(bdiff / 8 ))))){
 			/* resize gloopstart */
 			loopview->grabsel = 1; /* grab the start */
 			cursor = gdk_cursor_new(GDK_LEFT_SIDE);
 			gdk_window_set_cursor(widget->window,cursor);
-		}else if (((ediff < maxd) && (ediff > 0)) ||
-			((ediff < 0) && (ediff  > (maxd * -1)
+		}else if (((ediff < maxd) && (ediff >= 0)) ||
+			((ediff <= 0) && (ediff  > (maxd * -1)
 				&& (sdiff <(bdiff / 8 ))))){
 			/* resize gloopend */
 			cursor = gdk_cursor_new(GDK_RIGHT_SIDE);
@@ -1199,8 +1247,8 @@ static gboolean gtk_loopview_button_press( GtkWidget *widget,
 	} else if ((event->button == 3) && (event->state & GDK_CONTROL_MASK)){
 		if (loopview->looperdata->buf){
 			looperdata_lock(loopview->looperdata);
-			loopview->looperdata->recpos =
-				gtk_loopview_graphical2sample(loopview,event->x);
+			looperdata_set_recpos(loopview->looperdata, 
+				(double)gtk_loopview_graphical2sample(loopview,event->x));
 			looperdata_unlock(loopview->looperdata);
 		}
 	} else if (event->button == 3){
@@ -1322,12 +1370,15 @@ static gboolean gtk_loopview_button_release( GtkWidget      *widget,
 				loopview->grecstart  = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recstart);
 				loopview->grecend    = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recend);
 /*				gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
-				peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, loopview->npeaks);
+				if (!loopview->suspended)
+					peakview_set_area(loopview->peakview,loopview->zoomstart, 
+							loopview->zoomend, loopview->npeaks);
 			}
 			loopview->gzoomstart = 0;
 			loopview->gzoomend = 0;
 /*			gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
-			peakview_calculate(loopview->peakview, loopview->zoomstart, loopview->zoomend);
+			if (!loopview->suspended)
+				peakview_calculate(loopview->peakview, loopview->zoomstart, loopview->zoomend);
 /*			gtk_widget_draw (GTK_WIDGET(loopview),NULL);*/
 		}
 
