@@ -19,6 +19,7 @@
 #include "configfile.h"
 #include "sfpreview.h"
 #include "../../common/speciallist.h"
+#include "../../common/clickmodes.h"
 
 #ifdef HAVE_ALSA
 #include "../../common/midi.h"
@@ -76,6 +77,7 @@ char *txt = NULL;
 
 looper_list_t *looperlist;
 speciallist_t *looperdatalist;
+speciallist_t *clickmode_list;
 
 int looperindexcounter = 0;
 int bufferindexcounter = 0;
@@ -99,22 +101,6 @@ osc_info_t *osc_info;
 #endif
 
 /*------- EVENT FUNCTIONS -------*/
-static gboolean delete_event( GtkWidget *widget,
-			      GdkEvent  *event,
-			      gpointer   data )
-{       
-/*
-    	g_print ("shutting down\n");
-*/
-/*	mixer_close(jack_info);*/
-	
-    /* Change TRUE to FALSE and the main window will be destroyed with
-     * a "delete_event". */
-	
-    	return FALSE;
-}	       
-	
-
 void set_status (const char* message){
 	char* mws;
 
@@ -132,9 +118,41 @@ void set_status (const char* message){
 }
 
 void destroy(GtkWidget *widget, gpointer data){
-	gtk_main_quit();
+	GtkWidget *dialog;
+        gint response;
+
+        dialog = gtk_message_dialog_new (GTK_WINDOW(window),
+                                GTK_DIALOG_DESTROY_WITH_PARENT,
+                                GTK_MESSAGE_QUESTION,
+                                GTK_BUTTONS_YES_NO,
+                                "are you sure you want to quit?\n");
+	gtk_dialog_add_button (GTK_DIALOG (dialog), "Maybe",1783);
+        response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+        gtk_widget_destroy (dialog);
+        if (response == GTK_RESPONSE_YES){
+		mixer_close(jack_info);
+		gtk_main_quit();
+        }else if (response == 1783){
+		int i = (int)(3.0*rand()/(RAND_MAX+1.0));
+/*		printf ("i:%d\n",i);*/
+		if (i == 1) gtk_main_quit();
+		else set_status("maybe not quitting");
+	}else set_status("not quitting");
 }       
 
+static gboolean delete_event( GtkWidget *widget,
+                              GdkEvent  *event,
+                              gpointer   data ){
+    	/* Change TRUE to FALSE and the main window will be destroyed with
+     	 a "delete_event". */
+
+        destroy(GTK_WIDGET(widget),data);
+
+        return TRUE;
+}
+
+
 static void close_all_loopers (){
 	GList *tmpptr = g_list_first (looperlist->listptr);
 	looper_gui_t *looper = NULL;
@@ -301,6 +319,8 @@ gint main_idlefunc(gpointer data){
 		confirm_error(window,"jack kicked us out.\n please save and restart.\n");
 	}
 
+	if (!alive) set_status ("no jack - no playback !");
+
 	while (buf){
 		if (buf->pending == BUFFER_STATUS_READY){
 			gtk_progress_bar_update ((GtkProgressBar*)pbar,buf->percent_filled);
@@ -347,19 +367,23 @@ gint main_idlefunc(gpointer data){
 
 	if (countdown) countdown--;	
 	else if (loadfilestate == 1){
-                gint x,y,w,h;
+/*                gint x,y,w,h;*/
 
 /*		printf ("stage 1\n");*/
+/*
                 config_get_geometry(ch, &x,&y,&w,&h);
                 gtk_window_resize((GtkWindow*)window,w,h);
                 gtk_window_move((GtkWindow*)window,x,y);
+*/
 
 		while ((ti = config_next_buffer(ch)) != 0){
 			if (bufferindexcounter < ti) bufferindexcounter = ti;
 		       	buf = buffer_new(ti,jack_info->samplerate);
 			config_read_buffer (ch,buf);
 /*			printf ("buffer nr:%d created\n",ti);*/
-			speciallist_append(buffer_list,buf);
+			if (buffer_get_status(buf) != BUFFER_STATUS_DEAD)
+				speciallist_append(buffer_list,buf);
+			else confirm_error(window,"could not load %s",buf->filename);
 		}
 
 		buf = speciallist_get_first(buffer_list);
@@ -369,10 +393,12 @@ gint main_idlefunc(gpointer data){
 
 		loadfilestate++;
 	}else if (!not_ready && (loadfilestate == 2)){
+		gint x,y,w,h;
 		while ((ti = config_next_looper(ch)) != 0){
 /*			printf ("new looper:%d\n",ti);*/
 			if (looperindexcounter < ti) looperindexcounter = ti;
-			looper = looper_new(window, looperdatalist, ti);
+			looper = looper_new(window, globalbox, looperdatalist, clickmode_list, ti, 
+					config_get_looper_minimized (ch, ti));
 			if (jack_info->samplerate)
 				looper_set_samplerate(looper,jack_info->samplerate);
 			looper_list_append_looper(looperlist, looper);
@@ -380,6 +406,9 @@ gint main_idlefunc(gpointer data){
 			gtk_box_pack_start (GTK_BOX (globalbox), looper->mainbox, FALSE, FALSE, 0);
 			looper_update_bufferlist(looper,buffer_list);
 		}
+		config_get_geometry(ch, &x,&y,&w,&h);
+                gtk_window_resize((GtkWindow*)window,w,h);
+                gtk_window_move((GtkWindow*)window,x,y);
 /*		printf ("stage 2\n");*/
 		loadfilestate++;
 		countdown = 5; /* wait a while to settle widgets */
@@ -768,10 +797,6 @@ static void create_empty_buffer(gpointer data){
 
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){
 		buf = buffer_new(bufferindexcounter,jack_info->samplerate);
-		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stereo)))
-			buffer_set_channels(buf,1);
-		else
-			buffer_set_channels(buf,2);
 		buffer_lock(buf);
 		buf->pending = BUFFER_STATUS_LOADING;
 		if (jack_info->samplerate)
@@ -785,6 +810,9 @@ static void create_empty_buffer(gpointer data){
 			buffer_set_shortname(buf,"empty buffer");
 			buf->pending = BUFFER_STATUS_DEAD;
 		}
+		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stereo)))
+                        buffer_set_channels(buf,1);
+                else	buffer_set_channels(buf,2);
 		buffer_unlock(buf);
 
 		speciallist_append(buffer_list,buf);
@@ -794,7 +822,7 @@ static void create_empty_buffer(gpointer data){
 }
 	
 static void create_looper(gpointer data){	
-	looper_gui_t *loop = looper_new(window, looperdatalist, ++looperindexcounter);
+	looper_gui_t *loop = looper_new(window, globalbox, looperdatalist, clickmode_list, ++looperindexcounter, 0);
 /*	printf ("counter:%d\n",looperindexcounter);*/
 
 	if (jack_info->samplerate)
@@ -840,13 +868,13 @@ static void create_mainlayout(){
 	status = gtk_statusbar_new();
 	gtk_statusbar_set_has_resize_grip((GtkStatusbar*)status, TRUE);
 	gtk_box_pack_start (GTK_BOX (footerbox),status, TRUE, TRUE, 0);
-	gtk_widget_set_size_request (status,250,10);
+	gtk_widget_set_size_request (status,200,10); /* was 250,10 */
 	gtk_widget_show (status);	
 	context = gtk_statusbar_get_context_id ((GtkStatusbar*)status,"mainwindow");
 
 	pbar = gtk_progress_bar_new();
 	gtk_box_pack_start (GTK_BOX (footerbox),pbar, TRUE, TRUE, 0);
-	gtk_widget_set_size_request (pbar,400,10);
+	gtk_widget_set_size_request (pbar,150,10); /* was 400,10 */
 	gtk_widget_show (pbar);
 
 	gtk_widget_show (footerbox);
@@ -885,9 +913,8 @@ static void create_mainlayout(){
 
 	menu_items = gtk_menu_item_new_with_label ("quit");
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);
-
 	g_signal_connect_swapped (G_OBJECT (menu_items), "activate",
-				      G_CALLBACK (gtk_main_quit), 
+				      G_CALLBACK (destroy), 
 				      (gpointer) "quit");
 	gtk_widget_show (menu_items);
 
@@ -1021,7 +1048,7 @@ int main(int argc, char *argv[])
 	gtk_window_set_title (GTK_WINDOW (window), jack_info->name);
 	g_signal_connect(window, "destroy",G_CALLBACK(destroy), NULL);
 	g_signal_connect (G_OBJECT (window), "delete_event",G_CALLBACK (delete_event), NULL);
-	gtk_container_set_border_width(GTK_CONTAINER (window), 10);
+	/*gtk_container_set_border_width(GTK_CONTAINER (window), 10);*/
 
 	create_mainlayout();
 	
@@ -1040,7 +1067,9 @@ int main(int argc, char *argv[])
 		jack_info->samplerate = 44100;
 	}
 	
-	/* should deal with jack_info->alive here */
+	if (!jack_info->alive) alive = 0;
+
+	clickmode_list = clickmodes_create(jack_info->samplerate);
 
 	/* idle callback looks for changes in buffers and loopers */		
 /*	gtk_timeout_add(40,(GtkFunction)main_idlefunc,NULL);*/
