@@ -24,10 +24,13 @@ typedef struct _looper_gui {
 	int		id;
 	int		gid;
 	GtkWidget	*parent;
+	GtkWidget	*parentcontainer;
+	int 		vposition;
 	long		samplerate;
 /*	GList		*bufferlist;*/
 	speciallist_t   *bufferlist;
 	speciallist_t	*looperdatalist;
+	speciallist_t	*clickmodes;
 
 	/* for avoiding gui-loopbacks */
 	gboolean 	loopstartblock;
@@ -80,6 +83,10 @@ typedef struct _looper_gui {
 	GtkWidget	*normbutton;		/* button */
 	GtkWidget	*shuttlebutton;		/* toggle_button */
 	GtkWidget	*minimizebutton;	/* toggle_button */
+	GtkWidget	*upbutton;		/* button */
+	GtkWidget	*downbutton;		/* button */
+	GtkWidget	*upimage;		/* image.*/
+	GtkWidget	*downimage;		/* image */
 	GtkWidget	*playimage;		/* image */
 	GtkWidget	*recimage;		/* image */
 
@@ -100,6 +107,7 @@ typedef struct _looper_gui {
 	GtkWidget	*recendspin;		/* spinbutton */
 	GtkWidget	*reclengthspin;		/* spinbutton */
 	GtkWidget	*speedspin;		/* spinbutton */
+	GtkWidget	*diversityspin;		/* spinbutton */
 	GtkWidget	*densityspin;		/* spinbutton */
 	GtkWidget	*panswingspin;		/* spinbutton */
 
@@ -108,19 +116,24 @@ typedef struct _looper_gui {
 	
 } looper_gui_t;
 
-looper_gui_t* looper_new(GtkWidget* parent, speciallist_t *looperdatalist, int id);
+looper_gui_t* looper_new(GtkWidget* parent, GtkWidget* parentcontainer, speciallist_t *looperdatalist, 
+				speciallist_t *clickmodes, int id, int minimized);
 int looper_update_bufferlist(looper_gui_t* loop, speciallist_t *bufferlist);
 int looper_update_loopername(gchar* name);
 void looper_update_cursorposition (looper_gui_t *looper);
 void looper_update_widgets(looper_gui_t *looper);
 void looper_set_samplerate (looper_gui_t *looper,long samplerate);
 void looper_set_buffer_by_id (looper_gui_t *looper, int id);
+void looper_suspend_gui (looper_gui_t *looper, int suspend);
 
 looper_list_t* looper_list_new();
 void looper_close (looper_gui_t *looper);
 looper_list_t* looper_list_append_looper(looper_list_t* list, looper_gui_t *loop);
 looper_list_t* looper_list_remove_looper(looper_list_t* list, looper_gui_t *loop);
 
+int looper_get_position(looper_gui_t *loop);
+void looper_set_position(looper_gui_t *loop, int pos);
+
 
 /* LATER*/
 looper_list_t* looper_list_delete(looper_list_t* list);
